#include "fl_contrl_sys.h"
#include "p_blackboard.h"

#include "Handling_Control_alg.h"
#include "Pitch_Stabilization_alg.h"
#include "Actuator_Synchronization_alg.h"
#include "Actuator_Controller1_alg.h"
#include "Actuator_Controller2_alg.h"
#include "Actuator_Dynamics_alg.h"
#include "Aircraft_Dynamics_alg.h"
#include "Pilot_Steering_alg.h"


// Reference to Process table (fl_control_sys.c):
extern PROCESS_ATTRIBUTE_TYPE proc_table[NO_OF_PROCESSES];

// Reference to task ID table (fl_control_sys.c):
extern int TaskID[NO_OF_PROCESSES];

// Reference to watchdog id table (fl_control_sys.c):
extern WDOG_ID WatchdogID[NO_OF_PROCESSES];

// Reference to Data structure for exchanging data via p_balackboards:
extern P_BALCKBOARD_DATA blackboard_table[NO_OF_P_BLACKBOARDS];

////////////////////////////////////////////////////////////////////////
// Handling_Control (HC)														  //
////////////////////////////////////////////////////////////////////////

void Handling_Control_wd_fun(int a)
{
	taskResume(TaskID[T_HC]);
}

void Handling_Control(void)
{
	float Up=0.0,	//in
	      THM=0.0,	//in
	      Upf=0.0,	//out
	      THD=0.0;	//out
	
	//calculate:
	//printf("Hello task %d\n",T_HC);
	while(1)
		{
			if (wdStart (	WatchdogID[T_HC], 
							proc_table[T_HC].PERIOD, 
							(FUNCPTR)Handling_Control_wd_fun,
						1) == ERROR)
				{
					printf("Watchdog %d Error",T_HC);
					exit(0);
				}
		/////////////////////////////////////////////
		//	CALCULATE ALGORITHM:				   //
		/////////////////////////////////////////////
			{	int err_;
				//collect inputs:
				P_READ_BLACKBOARD(B_Up,&Up);
				P_READ_BLACKBOARD(B_THM,&THM);
				
				//	CALCULATE ALGORITHM:
				err_ = Handling_Control_alg(Up,THM,&Upf,&THD);
				
				//send outputs:
				P_DISPLAY_BLACKBOARD(B_Upf,&Upf);
				P_DISPLAY_BLACKBOARD(B_THD,&THD);
			}
		/////////////////////////////////////////////
			
			taskSuspend(0);
		}
}
//-----------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////
// Pitch_Stabilization (PS)													  //
////////////////////////////////////////////////////////////////////////
void Pitch_Stabilization_wd_fun(int a)
{
	taskResume(TaskID[T_PS]);
}

void Pitch_Stabilization(void)
{
	float Upf=0.0, 	//in
		  THD=0.0,	//in
		  THM=0.0,	//in
		  qm=0.0,	//in
		  Dhm=0.0,	//in 
		  Dhd=0.0;	//out
			
	//printf("Hello task %d\n",T_PS);
			while(1)
			{
				if (wdStart (	WatchdogID[T_PS], 
								proc_table[T_PS].PERIOD, 
								(FUNCPTR)Pitch_Stabilization_wd_fun,
							1) == ERROR)
					{
						printf("Watchdog %d Error",T_PS);
						exit(0);
					}
				/////////////////////////////////////////////
				//	CALCULATE ALGORITHM:				   //
				/////////////////////////////////////////////
				{	int err_;
					
					//collect inputs:
					P_READ_BLACKBOARD(B_THD,&THD);
					P_READ_BLACKBOARD(B_Upf,&Upf);
					P_READ_BLACKBOARD(B_THM,&THM);
					P_READ_BLACKBOARD(B_qm,&qm);
					P_READ_BLACKBOARD(B_Dhm,&Dhm);
				
					//calculate:
					err_ = Pitch_Stabilization_alg(THD,Upf,THM,qm,Dhm,&Dhd);
					
					//send outputs:
					P_DISPLAY_BLACKBOARD(B_Dhd,&Dhd);
				}
			/////////////////////////////////////////////
				
				taskSuspend(0);
			}
}
//----------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////
// Actuator_Synchronizatio	(AS)													  //
////////////////////////////////////////////////////////////////////////
void Actuator_Synchronization_wd_fun(int a)
{
	taskResume(TaskID[T_AS]);
}

void Actuator_Synchronization(void)
{	float Dhd=0.0,  //in
		  MA1=0.0,  //in
		  MA2=0.0,  //in
		  Dhd1=0.0, //out
		  Dhd2=0.0; //out
	
	//printf("Hello task %d\n",T_AS);
	while(1)
	{
		if (wdStart (	WatchdogID[T_AS], 
						proc_table[T_AS].PERIOD, 
						(FUNCPTR)Actuator_Synchronization_wd_fun,
					1) == ERROR)
		{
			printf("Watchdog %d Error",T_AS);
			exit(0);
		}
		/////////////////////////////////////////////
		//	CALCULATE ALGORITHM:				   //
		/////////////////////////////////////////////
		{	int err_;
			//collect inputs:
			P_READ_BLACKBOARD(B_Dhd,&Dhd);
			P_READ_BLACKBOARD(B_MA1,&MA1);
			P_READ_BLACKBOARD(B_MA2,&MA2);
			//calculate:
			err_ = Actuator_Synchronization_alg(Dhd,MA1,MA2,&Dhd1,&Dhd2);
			
			//send outputs:	
			P_DISPLAY_BLACKBOARD(B_Dhd1,&Dhd1);
			P_DISPLAY_BLACKBOARD(B_Dhd2,&Dhd2);
		}
		//-------------------------------------------
					
		taskSuspend(0);
	}
}

////////////////////////////////////////////////////////////////////////
// Actuator_Controller1 (AC1)														  //
////////////////////////////////////////////////////////////////////////
void Actuator_Controller1_wd_fun(int a)
{
	taskResume(TaskID[ T_AC1]);
}
void Actuator_Controller1(void)
{	
	float  Dhd1=0.0, //in
		   p1=0.0;   //out
	//printf("Hello task %d\n",T_AC1);
		while(1)
		{
			if (wdStart (	WatchdogID[T_AC1], 
							proc_table[T_AC1].PERIOD, 
							(FUNCPTR)Actuator_Controller1_wd_fun,
						1) == ERROR)
			{
				printf("Watchdog %d Error",T_AC1);
				exit(0);
			}
			/////////////////////////////////////////////
			//	CALCULATE ALGORITHM:				   //
			/////////////////////////////////////////////
			{	int err_;
				//collect inputs:
				P_READ_BLACKBOARD(B_Dhd1,&Dhd1);
				
				//calculate:
				err_ = Actuator_Controller1_alg(Dhd1,&p1);
				
				//send outputs:
				P_DISPLAY_BLACKBOARD(B_p1,&p1);
			}
		//-------------------------------------------
						
			taskSuspend(0);
		}	
}


////////////////////////////////////////////////////////////////////////
// Actuator_Controller2	 (AC2)													  //
////////////////////////////////////////////////////////////////////////
void Actuator_Controller2_wd_fun(int a)
{
	taskResume(TaskID[T_AC2]);
}
void Actuator_Controller2(void)
{
	float  Dhd2=0.0, //in
		   p2=0.0;   //out
	//printf("Hello task %d\n",T_AC2);
	while(1)
		{
			if (wdStart (	WatchdogID[T_AC2], 
							proc_table[T_AC2].PERIOD, 
							(FUNCPTR)Actuator_Controller2_wd_fun,
						1) == ERROR)
			{
				printf("Watchdog %d Error",T_AC2);
				exit(0);
			}
			/////////////////////////////////////////////
			//	CALCULATE ALGORITHM:				   //
			/////////////////////////////////////////////
			{	int err_;
				P_READ_BLACKBOARD(B_Dhd2,&Dhd2);
							
				//calculate:
				err_ = Actuator_Controller2_alg(Dhd2,&p2);
							
				//send outputs:
				P_DISPLAY_BLACKBOARD(B_p2,&p2);
			}
		//-------------------------------------------
						
			taskSuspend(0);
		}	
}

//void MW1(void)
//{}

//void MW2(void)
//{}

////////////////////////////////////////////////////////////////////////
// Actuator_Dynamics (AR)												  //
////////////////////////////////////////////////////////////////////////
void Actuator_Dynamics_wd_fun(int a)
{
	taskResume(TaskID[T_AR]);
}

void Actuator_Dynamics(void)
{
	float p1=0.0,	//in
		  p2=0.0,	//in
		  Dhm=0.0,	//out
		  MA1=0.0,	//out
		  MA2=0.0;	//out
		  
	//printf("Hello task %d\n",T_AR);
	while(1)
	{
		if (wdStart (	WatchdogID[T_AR], 
						proc_table[T_AR].PERIOD, 
						(FUNCPTR)Actuator_Dynamics_wd_fun,
					1) == ERROR)
		{
			printf("Watchdog %d Error",T_AR);
			exit(0);
		}
		/////////////////////////////////////////////
		//	CALCULATE ALGORITHM:				   //
		/////////////////////////////////////////////
		{	int err_;
			//collect inputs:
			P_READ_BLACKBOARD(B_p1,&p1);
			P_READ_BLACKBOARD(B_p2,&p2);
			//calculate:
			err_ = Actuator_Dynamics_alg(p1,p2,&Dhm,&MA1,&MA2);
			
			//send outputs:
			P_DISPLAY_BLACKBOARD(B_Dhm,&Dhm);
			P_DISPLAY_BLACKBOARD(B_MA1,&MA1);
			P_DISPLAY_BLACKBOARD(B_MA2,&MA2);
		}
	//-------------------------------------------				
		taskSuspend(0);
	}	
}

////////////////////////////////////////////////////////////////////////
// Aircraft_Dynamics (DA)											  //
////////////////////////////////////////////////////////////////////////
void Aircraft_Dynamics_wd_fun(int a)
{
	taskResume(TaskID[T_DA]);
}

void Aircraft_Dynamics(void)
{
	float 	Dhm=0.0, //in
			THM=0.0, //out
			qm=0.0;  //out
	//printf("Hello task %d\n",T_DA);
	while(1)
	{
		if (wdStart (	WatchdogID[T_DA], 
						proc_table[T_DA].PERIOD, 
						(FUNCPTR)Aircraft_Dynamics_wd_fun,
					1) == ERROR)
		{
			printf("Watchdog %d Error",T_DA);
			exit(0);
		}
		/////////////////////////////////////////////
		//	CALCULATE ALGORITHM:				   //
		/////////////////////////////////////////////
		{	int err_;
			//collect inputs:
			P_READ_BLACKBOARD(B_Dhm,&Dhm);
			
			//calculate:
			err_ = Aircraft_Dynamics_alg(Dhm,&THM,&qm);
			
			//send outputs:
			P_DISPLAY_BLACKBOARD(B_THM,&THM);
			P_DISPLAY_BLACKBOARD(B_qm,&qm);
		}
	//-------------------------------------------				
		taskSuspend(0);
	}		
}


////////////////////////////////////////////////////////////////////////
//Pilot_Steering (PT)										  		  //
////////////////////////////////////////////////////////////////////////
void Pilot_Steering_wd_fun(int a)
{
	taskResume(TaskID[T_PT]);
}
void Pilot_Steering(void)
{
	float Up=0.0;
	//printf("Hello task %d\n",T_PT);
	while(1)
	{
		if (wdStart (	WatchdogID[T_PT], 
						proc_table[T_PT].PERIOD, 
						(FUNCPTR)Pilot_Steering_wd_fun,
					1) == ERROR)
		{
			printf("Watchdog %d Error",T_PT);
			exit(0);
		}
		/////////////////////////////////////////////
		//	CALCULATE ALGORITHM:				   //
		/////////////////////////////////////////////
		{	int err_;
			//collect inputs:
		
			//CALCULATE ALGORITHM:
			err_ = Pilot_Steering_alg(&Up);
		
			//send outputs:
			P_DISPLAY_BLACKBOARD(B_Up,&Up);
		}
	//-------------------------------------------				
		taskSuspend(0);
	}		
	
}